#!/usr/bin/env python

"""
A module providing elementary graph support for programs.

Copyright (C) 2008, 2009, 2010, 2011 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from micropython.data import Class, Function
from micropython.errors import TableError
import sys, os

def get_graph(program, out=None, filename=None):

    """
    Using the importer maintained by the given 'program', write out the graph
    definition for consumption by the Graphviz tools such as the dot program.

    If the optional 'out' parameter is set to a stream, that stream will be
    used to write out the graph definition; otherwise, standard output will be
    used.

    If the optional 'filename' is specified, the leafname minus extension will
    be used as the name of the graph.

    A command line like the following can be used to produce viewable graph
    files:

    dot -T svg -o output.svg output.dot
    """

    filename = filename and os.path.splitext(os.path.split(filename)[-1])[0] or "program"

    importer = program.get_importer()
    objtable = program.get_object_table()

    out = out or sys.stdout
    print >>out, 'digraph %s {' % filename
    print >>out, '  ratio=auto;'
    print >>out, '  center=true;'
    print >>out, '  rankdir=LR;'

    for from_name, attributes in importer.inferred_name_references.items():
        from_attribute = get_unit(objtable, from_name)
        if from_attribute:
            for objname, attrname in attributes:
                if not have_unit(objtable, objname, attrname):
                    continue
                colour = get_colour(from_name, objname, attrname, importer)
                print >>out, '  "%s" -> "%s.%s" [color="%s"];' % (from_name, objname, attrname, colour)

    print >>out, "}"

def get_colour(from_name, objname, attrname, importer):
    try:
        if (objname, attrname) in importer.specific_name_references[from_name]:
            return "red"
    except KeyError:
        pass
    return "black"

def get_unit(objtable, attribute):
    t = attribute.rsplit(".", 1)
    try:
        objname, attrname = t
        if not have_unit(objtable, objname, attrname, Function):
            return None
        return attribute
    except ValueError:
        return t[0]

def have_unit(objtable, objname, attrname, allowed_types=None):
    allowed_types = allowed_types or (Class, Function)
    try:
        attr = objtable.access(objname, attrname)
    except TableError:
        #print >>sys.stderr, "%s.%s not found!" % (objname, attrname)
        return 0

    for value in attr.get_values():
        if isinstance(value, allowed_types):
            return 1
    return 0

# vim: tabstop=4 expandtab shiftwidth=4
